<?php

namespace Concrete\Package\JlPackageMagicPrb;

use Concrete\Core\Package\Package;
use Concrete\Core\Package\PackageService;
use Concrete\Core\Support\Facade\Events;
use Concrete\Core\Support\Facade\Application;
use Illuminate\Filesystem\Filesystem;

use JtF\PackageMagic\PackageSources\Plugins\FromPRB;

defined('C5_EXECUTE') or die("Access Denied.");

/*
 * Package Magic PRB by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class Controller extends Package {

    protected $pkgHandle = 'jl_package_magic_prb';
    protected $appVersionRequired = '8.5.12';
    protected $pkgVersion = '9.0.0'; //

    public function getPackageDescription() {
        return t('Integrate a PRB source for packages to Package Magic. This source is only relevant to current PRB members. List PRB submissions and install them directly to your PRB test site. You will be required to copy two ConcreteCMS cookies from your browser developer console. These will saved encrypted.');
    }

    public function getPackageName() {
        return t('Package Magic PRB');
    }

    public function validate_install() {
        $pkg_m = $this->app->make(PackageService::class)->getByHandle('jl_package_magic_starter');
        if (!is_object($pkg_m)) {
            $err = Application::getFacadeApplication()->make('error');
            $err->add(t('Package Magic Starter must be installed before Package Magic PRB. This package provides an extension for Package Magic.'));
            return $err;
        }
    }

    public function install() {
        $pkg = parent::install();
        $this->do_install($pkg);
    }

    public function upgrade() {
        parent::upgrade();
        $this->do_install($this);
    }

    private function do_install($pkg) {
        /*
         * Sort out resources and Re-scan for Package Magic Source plugins
         */
        $app = Application::getFacadeApplication();
        $app->make('JtF\PackageMagic\PackageSources\PackageSourcesController')->listAll(true);
    }

    /*
     * Remove any cached PRB list or concrete5.org cookie details
     */

    public function uninstall() {
        $this->logout_concrete5();
        parent::uninstall();
        $app = Application::getFacadeApplication();
        $app->make('JtF\PackageMagic\PackageSources\PackageSourcesController')->listAll(true);
    }

    public function getPackageAutoloaderRegistries() {
        return ['src/JtF/PackageMagic' => '\\JtF\\PackageMagic'];
    }

    /*
     * Flush cookie cache file when cache is flushed
     */

    public function on_start() {
        Events::addListener('on_cache_flush', function () {
            $this->logout_concrete5();
        }
        );
    }

    /*
     * Clear all cookies and records, so forcing a logout
     */

    private function logout_concrete5() {
        FromPRB::logout_concreteCMS($this, null);
    }

}
