<?php

namespace JtF\PackageMagic\PackageSources\Plugins;

use Concrete\Core\Http\Request;
use Exception;
use Concrete\Core\Page\Page;
use JtF\PackageMagic\PackageSources\PackageSourcePluginBase;
use JtF\PackageMagic\PackageZipValidation;

defined('C5_EXECUTE') or die('Access Denied');

/*
 * Package Magic by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 */

class FromPRB extends PackageSourcePluginBase
{

    protected $curl_timeout = 20;

    public function getName()
    {
        return t('From PRB');
    }

    public function getDescription()
    {
        return t('Pick a submission from the PRB.');
    }

    public function getHelp()
    {
        return t('This source is only relevant to current PRB members. List PRB submissions and install them directly to your PRB test site. You will be required to copy two ConcreteCMS cookies from your browser developer console. These will saved encrypted.');
    }

    /*
     * Fragment for the edit form dialog
     *
     * https://stackoverflow.com/questions/20049393/using-php-curl-to-login-to-my-websites-form
     * http://blog.andyhunt.info/2011/12/21/using-php-and-curl-to-log-in-to-a-website/
     * https://stackoverflow.com/questions/3008817/login-to-remote-site-with-php-curl
     * https://www.sitepoint.com/using-curl-for-remote-requests/
     *
     * Now uses cookies copied across rather than actual login form
     *
     */

    public function edit($controller, $data)
    {
        $data = $this->synthesizeData($data);
        extract($controller->getSets());
        $form = $this->form();

        $pkg = $this->package('jl_package_magic_prb');
        $cfg = $pkg->getFileConfig();

        ob_start();
        $request = $this->app()->make(Request::class);
        if ($request->request->get('marketplace_cookie_code') && $request->request->get('community_cookie_code')) {
            $marketplace_cookie_code = $request->request->get('marketplace_cookie_code');
            $community_cookie_code = $request->request->get('community_cookie_code');
            $cfg->save('portland.marketplace_cookie_code', $this->encrypt($marketplace_cookie_code));
            $cfg->save('portland.community_cookie_code', $this->encrypt($community_cookie_code));
        } else {
            $enc_mpcc = $cfg->get('portland.marketplace_cookie_code');
            $enc_cmcc = $cfg->get('portland.marketplace_cookie_code');
            if (!empty($enc_mpcc) && !empty($enc_cmcc)) {
                $marketplace_cookie_code = $this->decrypt($enc_mpcc);
                $community_cookie_code = $this->decrypt($enc_cmcc);
            }
        }

        if ($this->is_force_login() || empty($marketplace_cookie_code) || empty($community_cookie_code)) {
            self::logout_concreteCMS($controller, $data);
            $this->clear_force_login();
            $submissions = false;
        } else {
            /*
             * TODO. For the future, either an option to show submissions
             * that have failed auto test, or merge the failed test list and
             * highlight status (red/green)
             */
            $submissions = $this->fetch_all_PRB_lists($controller, $data);
        }
        ?>
        <style>

            .from_prb_wrapper {
                margin-top: 10px;
            }

            .prb_hover_help {
                cursor: help;
            }
        </style>
        <div class="from_prb_wrapper">
            <?php
            if ($submissions === false) {

                ?>
                <legend>
                    <?php
                    echo t('Connect to ConcreteCMS Marketplace');
                    ?>
                </legend>
                <div class="alert alert-warning">
                    <?php
                    echo t("Login to the marketplace review page in your browser. Open your browser Dev Console to the Network/Doc tab and examine the cookies response. In Chrome you will need to <em>Show filtered out cookies</em>. Copy/paste the <em>CONCRETE5</em> cookies here, then Login.");
                    echo '<br>';
                    echo t("Do not remain logged in on a publicly available site. Your browser cookies are vulnerable and could be used to hijack your account");
                    ?>
                </div>
                <div class="fieldset" style="margin-bottom:1000px;">
                    <div class="form-group">
                        <?php
                        //$marketplace_cookie_code = $controller->global_settings->
                        echo $form->label('marketplace_cookie_code', 'marketplace.concretecms.com');
                        echo $form->text('marketplace_cookie_code', $marketplace_cookie_code??null,
                            [
                                    'placeholder' => t('CONCRETE5 cookie value')
                            ]);
                        ?>
                    </div>

                    <div class="form-group">
                        <?php
                        echo $form->label('community_cookie_code', 'community.concretecms.com');
                        echo $form->text('community_cookie_code', $community_cookie_code??null,
                            [
                                'placeholder' => t('CONCRETE5 cookie value')
                            ]);
                        ?>
                    </div>
                    <div class="form-group">
                        <?php
                        echo $form->submit('c5_submit', t('Login'), ['class' => 'btn btn-primary pull-right float-end']);
                        ?>
                    </div>
                </div>

                <?php
            } else {
                $this->clear_force_login();
                if (!empty($submissions)) {
                    /*
                     * TODO, sort/filter list. Maybe implement using same list mechanism as
                     * PM plugins, so also searchable.
                     */
                    ?>
                    <table class="table table-striped">
                        <thead>
                        <tr>
                            <th>

                            </th>
                            <th>

                            </th>
                            <th>
                                <?php
                                echo t('Item')
                                ?>
                            </th>
                            <th>
                                <?php
                                echo t('Linter')
                                ?>
                            </th>
                            <th>
                                <?php
                                echo t('Author')
                                ?>
                            </th>
                            <th>
                                <?php
                                echo t('Added on')
                                ?>
                            </th>
                            <th>
                                <?php
                                echo t('Last comment')
                                ?>
                            </th>
                            <th>
                                <?php
                                echo t('Will go live')
                                ?>
                            </th>
                            <th>
                                <?php
                                echo t('Last reply')
                                ?>
                            </th>
                        </tr>
                        </thead>
                        <tbody>
                        <?php
                        /*
                         * TODO, CHECK IF INSTALLED
                         * Something for the future. Not as simple as it may sound as
                         * submissions have names, but the list doesn't show handles,
                         * so could fail when submission names change.
                         */
                        foreach ($submissions as $submission) {
                            ?>
                            <tr data_page="<?php echo h($submission['submission_id']); ?>">
                                <td>
                                    <?php
                                    echo $form->radio($this->prefixNamespace('submission_id'), h($submission['submission_id']));
                                    ?>
                                </td>
                                <td>
                                    <?php
                                    if ($submission['type'] === 'Addon' || $submission['type'] === t('Addon')) {
                                        ?>
                                        <i class="fa fas fa-cog"></i>
                                        <?php
                                    } else {
                                        ?>
                                        <i class="fa fas fa-television fa-tv"></i>
                                        <?php
                                    }
                                    ?>
                                </td>
                                <td>
                                    <a href="<?php echo sprintf(self::REVIEW_PAGE, $submission['submission_id']); ?>"
                                       target="_blank" title="<?php echo t('Review thread PRB'); ?>"
                                       data-toggle="title">
                                        <?php
                                        echo $submission['name'];
                                        ?>
                                    </a>
                                </td>
                                <td>
                                    <?php
                                    if ($submission['linter_status'] === 'failed') {
                                        ?>
                                        <i class="text-danger fa fas fa-times-circle prb_hover_help"
                                           title="<?php echo t('Failed automated tests') ?>"></i>
                                        <?php
                                    } elseif ($submission['linter_status'] === 'testing') {
                                        ?>
                                        <i class="text-muted fa fas fa-question-circle prb_hover_help"
                                           title="<?php echo t('Tests in progress') ?>"></i>
                                        <?php
                                    } else {
                                        ?>
                                        <i class="text-success fa fas fa-check-circle prb_hover_help"
                                           title="<?php echo t('Passed automated tests') ?>"></i>
                                        <?php
                                    }
                                    ?>
                                </td>
                                <td>
                                    <?php
                                    echo $submission['developer_name'];
                                    ?>
                                </td>
                                <td>
                                    <?php
                                    echo $submission['submission_date']
                                    ?>
                                </td>
                                <td>
                                    <?php
                                    echo $submission['comment_date']
                                    ?>
                                </td>
                                <td>
                                    <?php
                                    echo $submission['time_remaining']
                                    ?>
                                </td>
                                <td>
                                    <?php
                                    echo $submission['comment_by']
                                    ?>
                                </td>
                            </tr>
                            <?php
                        }
                        ?>
                        </tbody>
                    </table>
                    <?php
                } else {
                    ?>
                    <div class="alert alert-danger">
                        <?php
                        echo t('The PRB list is empty!');
                        ?>
                    </div>
                    <?php
                }
                ?>
                <div class="form-group">
                    <a href="#c5_logout" class="btn btn-default btn-secondary pull-left float-start"><i class="fa fas fa-close fa-sign-out-alt"> </i>
                        <?php
                        echo t('Logout from ConcreteCMS Marketplace');
                        ?>
                    </a>
                    <a href="#c5_prb_refresh" class="btn btn-info pull-right float-end"><i class="fa fas fa-refresh fa-redo"> </i>
                        <?php
                        echo t('Refresh');
                        ?>
                    </a>
                </div>

                <?php
            }
            ?>
        </div>
        <?php
        $page = Page::getCurrentPage();
        $cID = $page->getCollectionID();
        $args = [
            'ccm_token'    => $this->make_token(),
            'called_class' => __CLASS__,
            'action'       => 'from_prb_refresh_ajax',
            'cID'          => $cID
        ];
        $args['sk'] = $this->encrypt($args);
        $url = self::app()->make('helper/url')->buildQuery($this->ajax_controller_url(), $args);
        $args2 = [
            'ccm_token'    => $this->make_token(),
            'called_class' => __CLASS__,
            'action'       => 'from_prb_logout_ajax',
            'cID'          => $cID
        ];
        $args2['sk'] = $this->encrypt($args2);
        $url2 = self::app()->make('helper/url')->buildQuery($this->ajax_controller_url(), $args2);
        ?>
        <script type="text/javascript">
            (function ($) {
                $(document).ready(function () {
                    var this_form = $('#package_magic_upload');

                    this_form.find('a[href="#c5_prb_refresh"]').one('click', function (ev) {
                        ev.preventDefault();
                        $.fn.dialog.showLoader();
                        $.ajax({
                            url: '<?php echo $url; ?>',
                            type: 'get',
                        }).always(function () {
                            $.fn.dialog.hideLoader();
                            window.location.reload();
                        });
                    });

                    this_form.find('a[href="#c5_logout"]').one('click', function (ev) {
                        ev.preventDefault();
                        $.fn.dialog.showLoader();
                        $.ajax({
                            url: '<?php echo $url2; ?>',
                            type: 'get',
                        }).always(function () {
                            $.fn.dialog.hideLoader();
                            window.location.reload();
                        });
                    });
                });
            })(jQuery);
        </script>
        <?php
        $opts_HTML = ob_get_contents();
        ob_end_clean();
        return $opts_HTML;
    }

    public function from_prb_refresh_ajax($controller, $data)
    {
        $cache = self::app()->make('cache/expensive');
        $cache->delete(self::CACHE_KEY);
        $cache->delete(self::CACHE_KEY . 'testing');
        $cache->delete(self::CACHE_KEY . 'failed');
        $ajax = self::app()->make('helper/ajax')->sendResult(['msg' => 'refresh']);
        exit;
    }

    public function from_prb_logout_ajax($controller, $data)
    {
        self::logout_concreteCMS($controller, $data);
        $this->set_force_login();
        self::app()->make('helper/ajax')->sendResult(['msg' => 'logout']);
        exit;
    }

    /*
     * Gets all prb lists, regular, testing and failed.
     */
    private function fetch_all_PRB_lists($controller, $data)
    {
        $submissions = $this->fetch_PRB_list($controller, $data);
        $submissions_testing = $this->fetch_PRB_list($controller, $data, 'testing');
        $submissions_failed = $this->fetch_PRB_list($controller, $data, 'failed');
        $submissions = array_merge($submissions, $submissions_testing, $submissions_failed);
        return $submissions;
    }

    /*
     * Gets the (cached) PRB list and scrapes it for submissions
     *
     * Any more complex and this would be easier with phpquery or even pass this on to the
     * view and use jQuery. However, the page is pretty much static in design and provides a
     * table, so can get away with quick and dirty regexes and strstr
     */

    private function fetch_PRB_list($controller, $data, $status = null)
    {
        $page = $this->fetch_PRB_list_cached_page($controller, $data, $status);
        if ($page !== false) {

            $p1 = strstr($page, 'id="prb"');
            $p1 = strstr($p1, '/thead>');
            $p1 = strstr($p1, '<tr>');
            $p1 = strstr($p1, '</table>', true);
            $rows = array_filter(explode('<tr>', $p1));
            if (empty($rows)) {
                return [];
            }

            $submission_rows = [];
            foreach ($rows as $row) {
                $cols = explode('<td>', $row);
                $info = [];

                if (strpos($cols[0], 'prb_add_on') !== false) {
                    $info['type'] = t('Addon');
                } else {
                    $info['type'] = t('Theme');
                }

                if (preg_match("/href=\".*?\/([0-9]*)\/\".*?\>/i", $cols[0], $m)) {
                    $info['submission_id'] = $m[1];
                }
                $info['name'] = trim(strip_tags($cols[0]));

                if (preg_match('/href="(.*?)"/i', $cols[1], $m)) {
                    $info['developer_page'] = $m[1];
                }
                $info['developer_name'] = trim(strip_tags($cols[1]));
                $info['submission_date'] = trim(strip_tags($cols[2]));
                $info['comment_date'] = trim(strip_tags($cols[3]));
                $info['time_remaining'] = trim(strip_tags($cols[4]));
                $info['comment_by'] = trim(strip_tags($cols[5]));
                $info['linter_status'] = $status;

                $submission_rows[$info['submission_id']] = $info;
            }
            if (!empty($submission_rows)) {
                return $submission_rows;
            }
            return [];
        }
        return [];
    }

    const MP_SITE = "https://marketplace.concretecms.com/";
    const CMS_SITE = "https://www.concretecms.com/";

    const CACHE_KEY = 'jl_package_magic/from_prb/fetch_PRB_list_page';
    const REVIEW_LIST = self::MP_SITE . "marketplace/review_pending/";

    const DOWNLOAD_PACKAGE = self::MP_SITE . 'marketplace/review_pending/-/download/%d/1/';
    const REVIEW_PAGE = self::MP_SITE . 'marketplace/review_pending/-/view_detail/%d/';

    private function cookie_file()
    {
        return DIR_CONFIG_SITE . '/jl_package_magic_concreteCMS_login_cookie.txt';
    }

    private function populate_cookie_from_browser()
    {

        //$marketplace_cookie_code = '58soeji3fi22ottrhdr62os7m7';
        //$community_cookie_code = '4fbfji000t3tpk48d23amjoa43';

        $pkg = $this->package('jl_package_magic_prb');
        $cfg = $pkg->getFileConfig();

        $enc_mpcc = $cfg->get('portland.marketplace_cookie_code');
        $enc_cmcc = $cfg->get('portland.marketplace_cookie_code');
        if (!empty($enc_mpcc) && !empty($enc_cmcc)) {
            $marketplace_cookie_code = $this->decrypt($enc_mpcc);
            $community_cookie_code = $this->decrypt($enc_cmcc);
        }

        $cookie_text[] =
            sprintf("#HttpOnly_marketplace.concretecms.com\tFALSE\t/\tFALSE\t0\tCONCRETE5\t%s",
                $marketplace_cookie_code);

        $cookie_text[] =
            sprintf("#HttpOnly_community.concretecms.com\tFALSE\t/\tFALSE\t0\tCONCRETE5\t%s",
                $community_cookie_code);

        $fh = $this->file();
        $fh->delete($this->cookie_file());
        $fh->append($this->cookie_file(), implode("\n", $cookie_text) . "\n");

        return;


        /*
         * An experiment - to read browser cookies from Chrome's database. Unfortunately
         * they are encrypted and I can't work out how to decrypt.
         * Knowledge on the web is out of date
         * * https://stackoverflow.com/questions/67979373/reading-and-decrypting-cookies-in-chrome-with-php


        $cookie_connection = [
            'url' => 'sqlite:////home/john/.config/google-chrome/Default/Cookies',
        ];
        $cookie_db = DriverManager::getConnection($cookie_connection);
        $c5_login_cookie_value = $cookie_db->fetchColumn("SELECT encrypted_value FROM `cookies` WHERE `name` = 'CONCRETE5' AND `host_key` = 'marketplace.concretecms.com'");
        */
    }

    /*
     * Using the core expensive cache. May develop my own tabled cache later.
     * A bit complex because it has to adapt (or at least not fail) when the
     * core api is continually farting about.
     */

    private function fetch_PRB_list_cached_page($controller, $data, $status = null)
    {
        $pkg = $this->package('jl_package_magic_prb');
        $cfg = $pkg->getFileConfig();

        $enc_mpcc = $cfg->get('portland.marketplace_cookie_code');
        $enc_cmcc = $cfg->get('portland.marketplace_cookie_code');
        if (!empty($enc_mpcc) && !empty($enc_cmcc)) {
            $marketplace_cookie_code = $this->decrypt($enc_mpcc);
            $community_cookie_code = $this->decrypt($enc_cmcc);
        }


        $cache = self::app()->make('cache/expensive');
        $r = $cache->getItem(self::CACHE_KEY . $status);
        if ((method_exists($r, 'isHit') && $r->isHit()) ||
            (method_exists($r, 'isMiss') && !$r->isMiss())
        ) {
            if (method_exists($r, 'get') && !empty($marketplace_cookie_code) && !empty($community_cookie_code)) {
                $page = $r->get();
                if (!empty($page)) { // Can Short-circuit with 0&& while testing
                    return $page;
                }
            }
        }

        $page = $this->fetch_PRB_list_page($controller, $data, $status);
        if (method_exists($r, 'set')) {
            if (method_exists($cache, 'save')) {
                $cache->save($r->set($page));
            } else {
                $r->set($page);
            }
        }
        return $page;
    }

    private function fetch_PRB_list_page($controller, $data, $status = null)
    {

        try {
            if ($status) {
                $page = $this->fetch_page(self::REVIEW_LIST . '?status=' . $status);
            } else {
                $page = $this->fetch_page(self::REVIEW_LIST);
            }
            /*
             * Simple quick-test we have a prb list
             */

            if (strpos($page, 'id="prb"') !== false) {
                return $page;
            }
        } catch (Exception $e) {
            ?>
            <div class="alert alert-danger">
                <?php
                echo $e->getMessage();
                ?>
            </div>
            <?php
            // its failed, so return false;
        }
        return false;
    }

    /*
    https://community.concretecms.com/oauth/2.0/authorize?type=web_server&client_id=0a47db2d2158bdde2f329287dab4a5effe8a244839047d12d57843d47f3d9203&redirect_uri=https%3A%2F%2Fwww.concretecms.com%2Fccm%2Fsystem%2Fauthentication%2Foauth2%2Fexternal_concrete5%2Fcallback&response_type=code&scope=openid
    */

    /*
     * Fragment for the edit form dialog
     *
     *
     * https://stackoverflow.com/questions/20049393/using-php-curl-to-login-to-my-websites-form
     * http://blog.andyhunt.info/2011/12/21/using-php-and-curl-to-log-in-to-a-website/
     * https://stackoverflow.com/questions/3008817/login-to-remote-site-with-php-curl
     * https://www.sitepoint.com/using-curl-for-remote-requests/
     *
     */

    private function fetch_page($url, $params = null)
    {
        $this->populate_cookie_from_browser();
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);

        if (!empty($params) && is_array($params)) {
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        }
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->curl_timeout);
        /*
         * Force curl to return rather than output
         */
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        /*
         * Follow redirects
         */
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);

        /*
         * Cookies from & to file
         */
        curl_setopt($ch, CURLOPT_COOKIEFILE, $this->cookie_file());
        curl_setopt($ch, CURLOPT_COOKIEJAR, $this->cookie_file());
        $response = curl_exec($ch);
        $error = curl_error($ch);
        curl_close($ch);
        if (!empty($error)) {
            $this->log('Error',t('CURL error fetching page %s, %s', $url, $error));
            $this->puke(t('CURL error fetching page %s, %s', $url, $error));
        }

        return $response;
    }

    /*
     * Header needed to get filename and check we have a zip
     * https://stackoverflow.com/questions/9183178/can-php-curl-retrieve-response-headers-and-body-in-a-single-request
     *
     * TODO - CACHE HERE - SHORT LIFE 10 mins?
     */

    private function fetch_download($id, $status = null)
    {
        $this->populate_cookie_from_browser();
        $url = sprintf(self::DOWNLOAD_PACKAGE, $id);
        $headers = [];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);

        curl_setopt($ch, CURLOPT_TIMEOUT, $this->curl_timeout);
        /*
         * Force curl to return rather than output
         */
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADERFUNCTION, function ($ch, $header) use (&$headers) {
            /*
             * $headers is updated and normaised.
             */
            $len = strlen($header);
            $header = explode(':', $header, 2);
            if (count($header) < 2) // ignore invalid headers
            {
                return $len;
            }

            $name = strtolower(trim($header[0]));
            if (!array_key_exists($name, $headers)) {
                $headers[$name] = [trim($header[1])];
            } else {
                $headers[$name][] = trim($header[1]);
            }
            return $len;
        }
        );
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);

        /*
         * Cookies from & to file
         */
        curl_setopt($ch, CURLOPT_COOKIEFILE, $this->cookie_file());
        curl_setopt($ch, CURLOPT_COOKIEJAR, $this->cookie_file());

        $response = curl_exec($ch);
        $error = curl_error($ch);
        curl_close($ch);
        if (!empty($error)) {
            $this->log('Error',t('CURL error fetching download %s, %s', $url, $error));
            $this->puke(t('CURL error fetching download %s, %s', $url, $error));
        }

        $header_err = $this->check_headers($headers);

        if (!empty($header_err)) {
            $this->log('Error',t('Header error fetching download %s, %s', $url, $error));
            $this->puke(t('Header error fetching download %s, %s', $url, $header_err));
        }
        return $response;
    }

    private $submission_id;
    private $submission_package_handle;

    private function check_headers($headers)
    {
        if (!empty($headers['content-disposition']) && is_array($headers['content-disposition'])) {
            $content_disposition = reset(array_filter($headers['content-disposition']));
            if (!empty($content_disposition)) {
                if (preg_match("/filename=\"(.*?)\.zip\"/", $content_disposition, $m)) {
                    $this->submission_package_handle = $m[1];
                    return;
                }
            }
        }
        /*
         * IF the above fails, log the headers for future diagnosis.
         */
        $this->log('Error', json_encode($headers, JSON_PRETTY_PRINT));
        return t('Download failed, you may need to logout and login again to get a clean connection.');
    }

    /*
     * After selecting a package, configure the plugin with a package it will
     * provide to the client. As the file manager may be storing the file remotely,
     * we get the content and save a temporary copy that can be deleted on cleanup.
     *
     * https://documentation.concrete5.org/developers/working-with-files-and-the-file-manager/accessing-the-physical-file-from-a-file-object
     *
     */

    public function initialiseSource($controller, $data)
    {
        $data = $this->synthesizeData($data);

        $submissions = $this->fetch_all_PRB_lists($controller, $data);

        extract($controller->getSets());
        $this->submission_id = $data['submission_id'];

        if (empty($this->submission_id)) {
            $this->puke(t('Please select a package from the PRB list'));
        }

        $submission = $submissions[$data['submission_id']??null]??null;
        if (empty($this->submission_id)) {
            $cache = self::app()->make('cache/expensive');
            $cache->delete(self::CACHE_KEY);
            $cache->delete(self::CACHE_KEY . 'testing');
            $cache->delete(self::CACHE_KEY . 'failed');
            $this->puke(t('Submission data is obsolete, please try again'));
        }

        $this->intermediate_file = $this->file()->getTemporaryDirectory() . DIRECTORY_SEPARATOR .
            $this->getClassName(__CLASS__) . '-intermediate-' . $this->submission_id . '.zip';

        try {
            if (file_exists($this->intermediate_file)) {
                $this->file()->delete($this->intermediate_file);
            }
        } catch (Exception $e) {
            $this->puke(t('Unable to delete intermediate file %s. %s', $this->intermediate_file, $e->getMessage()));
        }

        try {
            if (file_exists($this->intermediate_file)) {
                $this->puke(t('Unable to remove previous intermediate file %s', $this->intermediate_file));
            }
        } catch (Exception $e) {
            $this->puke(t('Unable to remove previous intermediate file %s. %s', $this->intermediate_file, $e->getMessage()));
        }

        /*
         * Get the file, from the 'package' url. Do not trim as this will corrupt the zip!!
         */
        $remote_content = $this->fetch_download($this->submission_id);

        if (empty($remote_content)) {
            $cache = self::app()->make('cache/expensive');
            $cache->delete(self::CACHE_KEY);
            $cache->delete(self::CACHE_KEY . 'testing');
            $cache->delete(self::CACHE_KEY . 'failed');
            $this->puke(t('Unable to retrieve submission %s', $submission['name']));
        }
        $this->file()->append($this->intermediate_file, $remote_content);
    }

    /*
     * No cleanup
     */

    /*
     * Validation specific to this source
     */

    public function validate()
    {
        if (empty($this->submission_id)) {
            return t('Please select a PRB submission.');
        }
        try {
            if (empty($this->intermediate_file) || !file_exists($this->intermediate_file)) {
                return t('Temporary intermediate file %s is not accessible.', $this->intermediate_file);
            }
        } catch (Exception $e) {
            return t('Unable to access intermediate file %s. %s', $this->intermediate_file, $e->getMessage);
        }


        if (filesize($this->intermediate_file) <= 0) {
            return t('File "%s" is empty', $this->submission_id);
        }
        $zip_val = new PackageZipValidation($this->intermediate_file);
        if (!$zip_val->is_valid_mime()) {
            return t('Source is not a zip archive');
        }
        return;
    }

    /*
     * A path to the zip file containing the package
     */

    public function getZipPath()
    {
        return $this->intermediate_file;
    }

    /*
     * The package handle
     */

    public function getPackageHandle()
    {
        if (!empty($this->submission_package_handle)) {
            return $this->submission_package_handle;
        }
        return $this->cleanHandle(basename($this->submission_id));
    }

    /*
     * Login action flag passed between requests by session
     */

    private function is_force_login()
    {
        $session = self::app()->make('session');
        $force_login = $session->get('jl_package_magic.upload.from_prb_logout');
        return !empty($force_login);
    }

    private function set_force_login()
    {
        $session = self::app()->make('session');
        $session->set('jl_package_magic.upload.from_prb_logout', true);
    }

    private function clear_force_login()
    {
        $session = self::app()->make('session');
        $session->set('jl_package_magic.upload.from_prb_logout', false);
    }

    /*
     * Clear all cookies and records, so forcing a logout
     */

    public static function logout_concreteCMS($controller, $data)
    {
        $cache = self::app()->make('cache/expensive');
        $cache->delete('jl_package_magic_from_prb_fetch_PRB_list_page');
        self::file_s()->delete(DIR_CONFIG_SITE . '/jl_package_magic_concreteCMS_login_cookie.txt');

        $pkg = self::package_s('jl_package_magic_prb');
        $cfg = $pkg->getFileConfig();

        $cfg->save('portland.marketplace_cookie_code', '');
        $cfg->save('portland.community_cookie_code', '');

    }

}
